#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import datetime
import logging
import os
import time
from urllib.request import urlopen, urlretrieve
from xml.dom import minidom

import dbus
from PIL import Image

import nfix_date as D
import nfix_image as IM
import nfix_radicale as R
import nfix_tools as T
from nfix_colors import clFarben

cAusrichtung = 'Ausrichtung'
cCaption = "wnfDesktopKalender 0.29"


class TwnfDesktopKalender:

    def __init__(self, aStichtag):
        self.IniDateiname = T.getHomeDirConfigWlsoft('wnfKontakt.ini')
        logging.basicConfig(filename=T.getHomeDirConfigWlsoft('nfix_desktop_kalender.log'),
                            format='%(asctime)s %(message)s',
                            level=logging.DEBUG)
        self.Con = None
        self.modus = 0
        self.heute = aStichtag
        self.jetzt = datetime.datetime.now()
        self.wochentag_i = D.wochentag_i(self.heute)
        self.von = D.ersterDieserWoche(self.heute)
        self.bis = self.von + datetime.timedelta(days=28)
        # print self.von,self.bis
        self.termine = {}
        self.caption = cCaption
        self.Breite = 1920
        self.Hoehe = 1080
        self.TagBreite = 130
        self.TagHoehe = 90
        self.TagRand = 2
        self.XOffset = self.Breite - 7 * self.TagBreite - 20
        self.YOffset = self.Hoehe - 4 * self.TagHoehe - 80
        self.HGColor = IM.clWhite;
        self.TextColor = IM.clBlack;
        self.LineColor = IM.clBlack;
        self.FarbeFT = clFarben['goldenrod'];
        self.FarbeHeute = clFarben['royalblue'];
        self.FarbeTransparent = IM.clTransparent;
        self.FarbeSA = clFarben['steelblue'];
        self.FarbeSO = clFarben['steelblue'];
        self.FarbeNormal = clFarben['cornflowerblue'];
        self.GrafikD = ""  # Grafikdateiname
        self.GrafikV = ""  # Grafikverzeichnis
        # self.TextFont = wnf_tools.cFontDejaVuSans
        # self.TextFontSize = 9
        self.TextFont = IM.cFontDejaVuSans
        self.TextFontSize = 12
        self.Bundesland = D.cSN
        self.CountDown = None
        # self.CountDown = D.strToDate('01.05.2024')
        # self.CountDown = D.strToDate('01.08.2023')
        # self.CountDown = D.strToDate('02.11.2020')
        self.CountDownATM = 0
        self.CountDownFont = IM.cFontDejaVuSans
        self.CountDownFontSize = 14
        self.FarbeCountDown = IM.clWhite;
        self.Position = 'or'
        self.setOffset()
        # self.TagHoehe = 5 * self.TextFontSize + self.CountDownFontSize
        logging.info('nfix_desktop_kalender initialisiert')
        self.RadicaleTermine = R.getTermineVonBis(self.von, self.bis)

    def setOffset(self):
        # print(self.Position)
        if self.Position == 'OL':
            self.XOffset = 20
            self.YOffset = 20
        elif self.Position == 'UL':
            self.XOffset = 20
            self.YOffset = self.Hoehe - 4 * self.TagHoehe - 80
        elif self.Position == 'OR':
            self.XOffset = self.Breite - 7 * self.TagBreite - 20
            self.YOffset = 20
        else:
            self.XOffset = self.Breite - 7 * self.TagBreite - 20
            self.YOffset = self.Hoehe - 4 * self.TagHoehe - 80

    def setAusrichtung(self, aAusrichtung):
        s, b = self.getHintergrundBild()
        if os.path.exists(s):
            ini, aIniDateiname = T.getIniDKWallpaper()
            T.schreibeString(ini, s, cAusrichtung, aAusrichtung)
            T.saveIni(ini, aIniDateiname)

    def getAusrichtung(self):
        s, b = self.getHintergrundBild()
        if os.path.exists(s):
            ini, _ = T.getIniDKWallpaper()
            s = T.leseString(ini, s, cAusrichtung)
            self.Position = s
            self.setOffset()
            return s
        else:
            return ''

    def eintragen(self, d, n, c):
        if n == '':
            return
        if d < self.von:
            return
        if d > self.bis:
            return
        t = self.termine.get(d)
        if t == None:
            t = []
        t.append((n, c))
        self.termine[d] = t
        # print(self.termine[d])

    def eintragenEinzeltermineR(self):
        for R in self.RadicaleTermine:
            print(R)
            if R[0] == R[1]:
                self.eintragen(R[0], R[2], IM.clTransparent)
        return

    def eintragenZeitraumRalt(self):
        for R in self.RadicaleTermine:
            print(R)
            if R[0] < R[1]:
                d = R[0]
                while (d <= self.bis) and (d <= R[1]):
                    self.eintragen(d, R[2], IM.clTransparent)
                    d = d + datetime.timedelta(1)
        return

    def eintragenZeitraumR(self):
        for R in self.RadicaleTermine:
            if R[0] < R[1]:
                g = R[0]
                while (g >= self.von) and (g <= self.bis) and (g <= R[1]):
                    n = R[2]
                    t = self.termine.get(g)
                    if t == None:
                        t = []
                    aAnz = len(t)
                    while len(t) < 4:
                        t.append(('', IM.clTransparent))
                    t[3]=(n, IM.clMoneyGreen2)
                    self.termine[g] = t
                    g = g + datetime.timedelta(1)

    def get_termin(self, d, i):
        s = ""
        cl = (0, 0, 0)
        t = self.termine.get(d)
        if t != None:
            j = 0;
            for n, c in t:
                if j == i:
                    s = n
                    cl = c
                    break
                j = j + 1
        return s, cl

    def getHintergrundBild(self):
        s, bez = downloadBingWallpaper('/tmp/')
        if T.wnfFileExists(s):
            return s, bez
        if self.GrafikV == '':
            return '', ''
        y = os.listdir(self.GrafikV)
        y.sort()
        # aus allen Dateien die jpg-bilder herausfiltern
        x = []
        for dateiname in y:
            if dateiname.lower().endswith(".jpg"):
                x.append(dateiname)
        i = D.tagesnummer(self.heute.year, self.heute.month, self.heute.day)
        # print i,x,y
        i = (i % len(x))  # Prozentzeichen bewirkt modulo
        s = x[i]
        s = self.GrafikV + s
        if not os.path.exists(s):
            s = self.GrafikD
        print('Hintergrundbild', s)
        return s, bez

    def zeichne_kalender(self, im, x0, y0, bez):
        d = self.von
        difference1 = datetime.timedelta(days=1)
        x = x0
        y = y0
        th = self.TextFontSize + 2
        w = 0
        for c in range(7):
            w = w + 1
            im.rect(x, y, self.TagBreite, th)
            s = D.cWochentageL[c]
            if (c == self.wochentag_i) and (self.FarbeHeute != self.FarbeTransparent):
                im.text_box_c(x + 1, y + 1, self.TagBreite - 2, th, s, self.FarbeHeute)
            elif (w == 7) and (self.FarbeSA != self.FarbeTransparent):
                im.text_box_c(x + 1, y + 1, self.TagBreite - 2, th, s, self.FarbeSO)
            elif (w == 6) and (self.FarbeSA != self.FarbeTransparent):
                im.text_box_c(x + 1, y + 1, self.TagBreite - 2, th, s, self.FarbeSA)
            elif (self.FarbeNormal != self.FarbeTransparent):
                im.text_box_c(x + 1, y + 1, self.TagBreite - 2, th, s, self.FarbeNormal)
            else:
                im.text_nc(x + 2, y + 2, self.TagBreite - 2, s)
            x = x + self.TagBreite + self.TagRand
        y = y0 + th + self.TagRand
        for z in range(4):
            x = x0
            w = 0
            for c in range(7):
                w = w + 1
                cl = self.FarbeTransparent
                if (d == self.heute):
                    cl = self.FarbeHeute
                elif (D.sFeiertag(d, self.Bundesland) != ""):
                    cl = self.FarbeFT
                elif (w == 7):
                    cl = self.FarbeSO
                elif (w == 6):
                    cl = self.FarbeSA
                else:
                    cl = self.FarbeNormal
                if (cl != self.FarbeTransparent):
                    im.rectclrand(x, y, self.TagBreite, self.TagHoehe, cl)
                else:
                    im.rect(x, y, self.TagBreite, self.TagHoehe)
                s = D.dateToStr(d)
                if (cl != self.FarbeTransparent):
                    im.text_box_c(x + 1, y + 2, self.TagBreite - 2, th, s, cl)
                else:
                    im.text_nc(x + 1, y + 2, self.TagBreite - 2, s)
                for i in range(4):
                    s, cl = self.get_termin(d, i)
                    if s != "":
                        print(d, s, cl)
                        if (cl != self.FarbeTransparent):
                            im.text_box_c(x + 1, y + 2 + (th * (i + 1)), self.TagBreite - 2, th, s, cl)
                        else:
                            im.text_nc(x + 1, y + 2 + (th * (i + 1)), self.TagBreite - 2, s)
                if self.CountDown:
                    if (self.CountDown >= self.heute) and (self.CountDown >= d) and (d >= self.heute):
                        cd = self.CountDown - d
                        s = str(cd.days)
                        im.text_countdown(x, y, self.TagBreite, self.TagHoehe, s, self.CountDownFont,
                                          self.CountDownFontSize, self.FarbeCountDown)
                d = d + difference1
                x = x + self.TagBreite + self.TagRand
            y = y + self.TagHoehe + self.TagRand

    #        s = datetime.datetime.ctime(self.jetzt)
        if bez!='':
            b, h = im.font_n.getsize(bez)
            b = 7 * self.TagBreite + 6 * self.TagRand
            cl = self.FarbeNormal
            im.text_box_c(x0, y, b, h, bez, cl)

    def setTimeStamp(self, im):
        # x, y, b, h, text, cl,
        s = '%s %s' % (self.caption, D.wnfJetztAsString())
        print(s)
        b, h = im.font_n.getsize(s)
        b += 20
        im.text_box_c(im.Breite - b, im.Hoehe - h, b, h, s, '#F5F5F5')
        # im.text_box_c(im.Breite - b, 0, b, h, s, '#F5F5F5')

    def ausgabe_png(self, dn):
        try:
            im = IM.TwnfImage(self.Breite, self.Hoehe, self.TextFont, self.TextFontSize,
                              self.HGColor, self.TextColor, self.LineColor)
            s, bez = self.getHintergrundBild()
            if s != "":
                im.hintergrundbild(0, 0, s)
            self.getAusrichtung()
            x0 = self.XOffset
            y0 = self.YOffset
            self.zeichne_kalender(im, x0, y0, bez)
            self.setTimeStamp(im)
            im.save_png(dn)
            # im.show()
            print("Wallpaper erzeugt: %s" % dn)
            logging.info("Wallpaper erzeugt: %s" % dn)
        except Exception as e:
            s = "Wallpaper nicht erzeugt: %s" % dn
            logging.error(s)
            print(e)
            logging.error(e)
            raise

    def ausgabe_png_transparent(self, dn):
        im = IM.TwnfImage(self.Breite, self.Hoehe, self.TextFont, self.TextFontSize, self.HGColor, self.TextColor,
                          self.LineColor)
        s, bez = self.getHintergrundBild()
        # if s != "":
        # im.hintergrundbild(0, 0, s)
        x0 = self.XOffset
        y0 = self.YOffset
        self.zeichne_kalender(im, x0, y0, bez)
        s, b = self.getHintergrundBild()
        if s != "":
            background = Image.open(s)
            foreground = im.im
            # pixeldata = list(foreground.getdata())
            # for i, pixel in enumerate(pixeldata):
            #    if pixel[:3] == (255, 255, 255):
            #        pixeldata[i] = (255, 255, 255, 0)
            # foreground.show()
            background.paste(foreground, (0, 0), foreground)
            background.save(dn)
            # background.show()
        else:
            im.save_png(dn)
            # im.show()

    def eintragenFeiertage(self):
        d = self.von
        while (d <= self.bis):
            n = D.sFeiertag(d, self.Bundesland)
            if n != "":
                self.eintragen(d, n, self.FarbeFT)
            d = d + datetime.timedelta(days=1)

    def eintragenIniM(self, aIni, aSection):
        if not aIni.has_section(aSection):
            return
        x = dict(aIni.items(aSection))
        for s in x.values():
            s = s.split(';')
            n = s[0]
            c = s[1]
            # m = s[2]
            t = s[3:]
            for vb in t:
                if vb != '':
                    vb = vb.split('-')
                    # print(t,vb)
                    v = D.strToDateTime(vb[0])
                    if len(vb) > 1:
                        b = D.strToDateTime(vb[1])
                    else:
                        b = None
                    print(v, b, n, c)
                    c = IM.pascalToRGB(c, IM.clTransparent)
                    if b:
                        d = v.date()
                        while (d <= self.bis) and (d <= b.date()):
                            self.eintragen(d, n, c)
                            d = d + datetime.timedelta(1)
                    else:
                        d = v.date()
                        self.eintragen(d, n, c)

    def eintragenIniT(self, aIni, aSection):
        if not aIni.has_section(aSection):
            return
        x = dict(aIni.items(aSection))
        for s in x.values():
            s = s.split(';')
            d = s[0]
            n = s[1]
            c = s[2]
            d = D.strToDateTime(d)
            # print(s, d, n, c)
            d = d.date()
            # print(n,d)
            c = IM.pascalToRGB(c, IM.clTransparent)
            self.eintragen(d, n, c)

    def eintragenIniG(self, aIni, aSection):
        if not aIni.has_section(aSection):
            return
        x = dict(aIni.items(aSection))
        for s in x.values():
            s = s.split(';')
            # Dieses Jahr
            d = s[0]
            n = s[1]
            c = s[2]
            d = D.strToDateTime(d)
            a = d
            d = D.diesesJahr(d)
            i = D.alterInJahren(a, d)
            n = "%d %s" % (i, n)
            d = d.date()
            # print(n,d)
            c = IM.pascalToRGB(c, IM.clTransparent)
            self.eintragen(d, n, c)
            # nächstes Jahr, falls der Kalender über den Jahreswechsel geht
            d = s[0]
            n = s[1]
            c = s[2]
            d = D.strToDateTime(d)
            a = d
            d = D.naechstesJahr(d)
            i = D.alterInJahren(a, d)
            n = "%d %s" % (i, n)
            d = d.date()
            # print(n,d)
            c = IM.pascalToRGB(c, IM.clTransparent)
            self.eintragen(d, n, c)

    def eintragenIniY(self, aIni, aSection):
        if not aIni.has_section(aSection):
            return
        x = dict(aIni.items(aSection))
        for s in x.values():
            s = s.split(';')
            d = s[0]
            t = int(s[1])
            n = s[2]
            c = s[3]
            d = D.strToDateTime(d)
            # print(s, d, n, c)
            # print(n,d)
            c = IM.pascalToRGB(c, IM.clTransparent)
            if t > 0:
                g = d.date()
                while g <= self.bis:
                    self.eintragen(g, n, c)
                    d = d + datetime.timedelta(t)
                    g = d.date()

    def eintragenIniZ(self, aIni, aSection):
        if not aIni.has_section(aSection):
            return
        x = dict(aIni.items(aSection))
        for s in x.values():
            s = s.split(';')
            v = s[0]
            b = s[1]
            n = s[2]
            c = s[3]
            v = D.strToDateTime(v)
            b = D.strToDateTime(b)
            # print(s, d, n, c)
            # print(n,d)
            c = IM.pascalToRGB(c, IM.clTransparent)
            d = v.date()
            while (d <= self.bis) and (d <= b.date()):
                self.eintragen(d, n, c)
                d = d + datetime.timedelta(1)

    def termineLesen(self):
        self.eintragenFeiertage()
        aIni, dateiname = T.getIniAnsi(T.cWnfDesktopKalenderIni)
        self.eintragenIniG(aIni, 'Geburtstage')
        self.eintragenIniT(aIni, 'Termine')
        self.eintragenEinzeltermineR()
        self.eintragenIniY(aIni, 'Zyklisch')
        self.eintragenIniM(aIni, 'Multi')
        self.eintragenIniZ(aIni, 'Zeitraum')
        self.eintragenZeitraumR()


def updateDesktopkalender():
    # Bugs
    # Plasma does not update the image of the same file name when it is replaced
    # deshalb hier zwei Bilder erzeugen
    d = datetime.date.today()
    # d = D.strToDate('01.07.2019')
    dk = TwnfDesktopKalender(d)
    dk.termineLesen()
    dk.GrafikV = os.path.join(T.wnfMeineBilder(), 'Wallpaper/')
    zdn1 = T.getHomeDirConfigWlsoftDir('nfix_desktopkalender', 'nfix_desktopkalender.png')
    dk.ausgabe_png(zdn1)
    zdn2 = T.getHomeDirConfigWlsoftDir('nfix_desktopkalender', 'nfix_desktopkalender_a.png')
    dk.ausgabe_png(zdn2)
    return zdn1, zdn2


def setWallpaper(aDateiname, plugin='org.kde.image'):
    jscript = """
    var allDesktops = desktops();
    print (allDesktops);
    for (i=0;i<allDesktops.length;i++) {
        d = allDesktops[i];
        d.wallpaperPlugin = "%s";
        d.currentConfigGroup = Array("Wallpaper", "%s", "General");
        d.writeConfig("Image", "file://%s")
    }
    """
    bus = dbus.SessionBus()
    plasma = dbus.Interface(bus.get_object('org.kde.plasmashell', '/PlasmaShell'), dbus_interface='org.kde.PlasmaShell')
    plasma.evaluateScript(jscript % (plugin, plugin, aDateiname))


def refreshDesktopkalender():
    d = datetime.date.today()
    # d = D.strToDate('01.07.2019')
    dk = TwnfDesktopKalender(d)
    dk.GrafikV = os.path.join(T.wnfMeineBilder(), 'Wallpaper/')
    zdn1, zdn2 = updateDesktopkalender()
    setWallpaper(zdn1)
    setWallpaper(zdn2)
    # T.gwenview(zdn)


def ausrichtungDesktopkalender(aAusrichtung):
    if not (aAusrichtung in ['OL', 'OR', 'UL', 'UR']):
        aAusrichtung = 'OL'
    d = datetime.date.today()
    # d = D.strToDate('01.07.2019')
    dk = TwnfDesktopKalender(d)
    dk.GrafikV = os.path.join(T.wnfMeineBilder(), 'Wallpaper/')
    dk.setAusrichtung(aAusrichtung)
    zdn1, zdn2 = updateDesktopkalender()
    setWallpaper(zdn1)
    setWallpaper(zdn2)
    # T.gwenview(zdn)


def join_path(*args):
    # Takes an list of values or multiple values and returns an valid path.
    if isinstance(args[0], list):
        path_list = args[0]
    else:
        path_list = args
    val = [str(v).strip(' ') for v in path_list]
    return os.path.normpath('/'.join(val))


def downloadBingWallpaper(aZielverzeichnis):
    # https://stackoverflow.com/questions/30173630/grab-bing-wallpaper-with-python3
    # Getting the XML File
    wartenaufnetzwerk = 0
    while wartenaufnetzwerk <= 10:
        try:
            idx = 0
            usock = urlopen(''.join(['http://www.bing.com/HPImageArchive.aspx?format=xml&idx=',
                                     str(idx),
                                     '&n=1&mkt=de-DE']))  # ru-RU, because they always have 1920x1200 resolution
            break
        except Exception as e:
            print('Error while downloading #', idx, e)
            if wartenaufnetzwerk >= 10:
                return '', ''
            else:
                wartenaufnetzwerk += 1
                time.sleep(1)

    try:
        xmldoc = minidom.parse(usock)

    # This is raised when there is trouble finding the image url.
    except Exception as e:
        print('Error while processing XML index #', idx, e)
        return
    # Parsing the XML File
    bezeichnung = ''
    for element in xmldoc.getElementsByTagName('copyright'):
        bezeichnung = element.firstChild.nodeValue
    with open('/tmp/dk_bilddestages.txt', 'w') as aDatei:
        aDatei.write(bezeichnung)
    for element in xmldoc.getElementsByTagName('url'):
        url = 'http://www.bing.com' + element.firstChild.nodeValue
        # Get Current Date as fileName for the downloaded Picture
        now = datetime.datetime.now()
        date = now - datetime.timedelta(days=int(idx))
        pic_path = join_path(aZielverzeichnis, ''.join([date.strftime('bing_wp_%d-%m-%Y'), '.jpg']))
        print(pic_path)
        if os.path.isfile(pic_path):
            print('Image of', date.strftime('%d-%m-%Y'), 'already downloaded.')
            return pic_path, bezeichnung
        print('Downloading: ', date.strftime('%d-%m-%Y'), 'index #', idx)
        # Download and Save the Picture
        # Get a higher resolution by replacing the file name
        urlretrieve(url.replace('_1366x768', '_1920x1200'), pic_path)
        return pic_path, bezeichnung


def gwenviewDesktopkalender():
    d = datetime.date.today()
    dk = TwnfDesktopKalender(d)
    dk.GrafikV = os.path.join(T.wnfMeineBilder(), 'Wallpaper/')
    zdn, b = dk.getHintergrundBild()
    T.gwenview(zdn)


def vorschauDesktopkalender():
    d = datetime.date.today()
    # d = D.strToDate('01.07.2019')
    dk = TwnfDesktopKalender(d)
    dk.termineLesen()
    # dk.GrafikV = '/wnfdaten/Bilder/Wallpaper/1920/'
    zdn1 = T.getHomeDirConfigWlsoftDir('nfix_desktopkalender', 'nfix_desktopkalender.png')
    dk.ausgabe_png(zdn1)
    T.gwenview(zdn1)
    return zdn1


if __name__ == "__main__":
    # vorschauDesktopkalender()
    # print(downloadBingWallpaper('/tmp/'))
    # updateDesktopkalender()
    refreshDesktopkalender()
    # setWallpaper('/home/wnf/.config/wlsoft/nfix_desktopkalender/nfix_desktopkalender_a.png')
