#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# PIL Handbuch http://www.pythonware.com/library/pil/handbook/index.htm
import os

from PIL import Image
from PIL import ImageDraw
from PIL import ImageFont

import nfix_tools as T

clBlack = (0, 0, 0)
clWhite = (255, 255, 255)
clRed = (255, 0, 0)
clGreen = (0, 255, 0)
clBlue = (0, 0, 255)
clGray = (100, 100, 100)
clYellow = (255, 255, 0)
clTuerkis = (0, 255, 255)
clLime = (0, 255, 0)
clAqua = (255, 255, 0)
clInfoBk = (255, 0, 255)

clMoneyGreen = (0xC0, 0xDC, 0xC0)
clMoneyGreen2 = (0x80, 0xFF, 0x80)
clMoneyGreen3 = (0x21, 0x5C, 0x2A)
clSkyBlue = (0xF0, 0xCA, 0xA6)
clCream = (0xF0, 0xFB, 0xFF)
clMediumGray = (0xA4, 0xA0, 0xA0)
clFuchsia = (0xFF, 0x00, 0xFF)
clTeal = (0x80, 0x80, 0x00)

clTransparent = (254, 254, 254)

clBlue2 = (0xB0, 0xB0, 0xB0)

cFontVerdana = "/usr/share/fonts/truetype/msttcorefonts/verdana.ttf"
cFontArial = "/usr/share/fonts/truetype/msttcorefonts/Arial.ttf"
cFontTimes = "/usr/share/fonts/truetype/msttcorefonts/times.ttf"
cFontDejaVuSans = "/usr/share/fonts/truetype/ttf-dejavu/DejaVuSans.ttf"
cFontLatoBold = "/usr/share/fonts/truetype/lato/Lato-Bold.ttf"
cFontLatoMedium = "/usr/share/fonts/truetype/lato/Lato-Medium.ttf"
cFontLatoLight = "/usr/share/fonts/truetype/lato/Lato-Light.ttf"


def pascalToRGB(s, aDefault):
    if type(s) is tuple:
        return s
    elif s == "clYellow":
        return clYellow
    elif s == "clLime":
        return clLime
    elif s == "clBlack":
        return clBlack
    elif s == "clWhite":
        return clWhite
    elif s == "clRed":
        return clRed
    elif s == "clBlue":
        return clBlue
    elif s == "clGreen":
        return clGreen
    elif s.startswith("$00"):
        b = int(s[3:5], 16)
        g = int(s[5:7], 16)
        r = int(s[7:9], 16)
        return (r, g, b)
    else:
        return aDefault


class TwnfImage:
    def __init__(self, aBreite, aHoehe, aFontname, aFontsize, aHGColor, aTextColor, aLineColor):
        self.Breite = aBreite
        self.Hoehe = aHoehe
        self.im = Image.new('RGBA', (aBreite, aHoehe))
        self.draw = ImageDraw.Draw(self.im)
        self.font_n_size = aFontsize
        if aFontname != "":
            self.font_n = ImageFont.truetype(aFontname, self.font_n_size)
        else:
            self.font_n = ImageFont.load_default()
        self.TextColor = aTextColor
        self.LineColor = aLineColor
        # weißen Hintergrund einstellen
        # self.draw.rectangle((0, 0, aBreite, aHoehe), fill=aHGColor)
        tw, self.TextHeight = self.font_n.getsize("Hg");

    def line(self, x1, y1, x2, y2):
        self.draw.line((x1, y1, x2, y2), fill=self.LineColor)

    def rect(self, x, y, b, h):
        self.line(x, y, x + b, y)
        self.line(x + b, y, x + b, y + h)
        self.line(x + b, y + h, x, y + h)
        self.line(x, y + h, x, y)

    def rectcl(self, x, y, b, h, cl):
        self.draw.rectangle((x, y, x + b, y + h), fill=cl)

    def rectclrand(self, x, y, b, h, cl):
        self.draw.rectangle((x, y, x + b, y + h), outline=self.LineColor, fill=cl)

    def show(self):
        self.im.show()
        return
        dn = T.tempDateiname(".png")
        self.im.save(dn)
        T.wnfDateiStarten(dn)
        os.remove(dn)

    def text(self, x, y, text, aFont, aSize, aWeight):
        if aFont == "":
            self.draw.text((x, y), text, fill=self.TextColor)
        else:
            self.draw.text((x, y), text, font=aFont, fill=self.TextColor)
        # self.draw.text((x,y),"Buß- und Bettag")
        # print text

    def text_cl(self, x, y, text, aFontname, aSize, cl):
        f = ImageFont.truetype(aFontname, aSize)
        if f == "":
            self.draw.text((x, y), text, fill=cl)
        else:
            self.draw.text((x, y), text, font=f, fill=cl)

    def text_n(self, x, y, text):
        self.text(x, y, text, self.font_n, self.font_n_size, "")

    def text_nc(self, x, y, b, text, halign="center"):
        tw, th = self.font_n.getsize(text);
        if halign == "center":
            leftStart = x + ((b - tw) / 2)
        elif halign == "right":
            leftStart = x + b - tw
        else:
            leftStart = x
        self.text_n(leftStart, y, text)

    def text_b(self, x, y, text):
        self.text(x, y, text, self.font_n, self.font_n_size, "bold")

    def text_box(self, x, y, b, h, text, cl):
        self.rectcl(x, y, b, h, cl)
        tw, th = self.font_n.getsize(text);
        # falls der Text nicht in die Textbox passt den string kürzen
        while tw > b - 4:
            text = text[:len(text) - 1]
            tw, th = self.font_n.getsize(text);
        self.text_n(x + 2, y, text)

    def text_box_c(self, x, y, b, h, text, cl, halign="center"):
        # Wenn der erste Buchstabe ein D,M ist, fehlt der erste Balken des Buchstabends
        # deshalb wird der String in zwei Leerzeichen gefasst
        text = ' ' + text + ' '
        self.rectcl(x, y, b, h, cl)
        tw, th = self.font_n.getsize(text);
        # falls der Text nicht in die Textbox passt den string kürzen
        while tw > b - 4:
            text = text[:len(text) - 1]
            tw, th = self.font_n.getsize(text);
        if halign == "center":
            leftStart = x + ((b - tw) / 2)
        elif halign == "right":
            leftStart = x + b - tw
        else:
            leftStart = x

        self.text_n(leftStart, y, text)

    def hintergrundbild(self, x, y, dn):
        try:
            # print(dn)
            hg = Image.open(dn)
            (b, h) = hg.size
            self.im.paste(hg, (x, y, b, h))
        except:
            print(dn, ' existiert nicht.')

    def save_png(self, dn):
        self.im.save(dn, "PNG")

    def save_jpg(self, dn):
        self.im.save(dn, "JPEG", quality=100)

    def text_countdown(self, x, y, b, h, text, aFontname, aFontSize, cl):
        # zeichnet den Text unten zentriert mit dem Font und in der Farbe
        if aFontname != "":
            f = ImageFont.truetype(aFontname, aFontSize)
        else:
            f = ImageFont.load_default()
        if f != "":
            tw, th = f.getsize(text);
            th = th + 4
            y = y + h - th
            x = x + ((b - tw) / 2)
            self.draw.text((x, y), text, font=f, fill=cl)


def nfCoronaBanner(aDateiname, aFarbe, aWert, aStandVom):
    im = TwnfImage(600, 88, cFontLatoBold, 24, clWhite, clBlack, clBlack)
    im.rectcl(0, 0, 600, 88, clWhite)
    x = 2
    y = 2
    h = 80
    b = 80
    im.rectcl(x, y, h, b, aFarbe)
    aText = str(aWert)
    f = ImageFont.truetype(cFontLatoBold, 32)
    if f != "":
        tw, th = f.getsize(aText);
        th = th + 4
        y = y + ((h - th) / 2)
        x = x + ((b - tw) / 2)
        im.draw.text((x, y), aText, font=f, fill=clWhite)
    x = b + 24
    y = 2
    aText = 'Corona 7-Tage-Inzidenz in Niederfrohna'
    f = ImageFont.truetype(cFontLatoLight, 24)
    if f != "":
        tw, th = f.getsize(aText);
        th = th + 6
        y = y + (h - th - th) / 2
        x = x
        im.draw.text((x, y), aText, font=f, fill=clBlack)
        y = y + th
        im.draw.text((x, y), aStandVom, font=f, fill=clBlack)
    if aDateiname != '':
        im.save_png(aDateiname)
    else:
        im.show()

def nfCoronaPrintlnXYB(im, f, x, y, b, aText):
    tw, th = f.getsize(aText);
    th = th + 6
    x = x + ((b - tw) / 2)
    im.draw.text((x, y), aText, font=f, fill=clBlack)
    y = y + th
    return y


def nfCoronaPrintln(im, f, b, y, aText):
    tw, th = f.getsize(aText);
    th = th + 6
    x = ((b - tw) / 2)
    im.draw.text((x, y), aText, font=f, fill=clBlack)
    y = y + th
    return y


def nfCoronaPrintln2(im, f, left, b, y, aText):
    tw, th = f.getsize(aText)
    th = th + 6
    x = left + ((b - tw) / 2)
    im.draw.text((x, y), aText, font=f, fill=clBlack)
    y = y + th
    return y


def coronaHochkannt(aDateiname, aFarbe, aWert, aFaelle7, aStandVom, aOrt):
    im = TwnfImage(204, 340, cFontLatoBold, 24, clWhite, clBlack, clBlack)
    im.rectcl(0, 0, 204, 408, clWhite)
    x = 2
    y = 2
    h = 200
    b = 200
    im.rectcl(x, y, h, b, aFarbe)
    aText = str(aWert)
    if int(aWert)>999:
        aFontsize = 84
    else:
        aFontsize = 96
    f = ImageFont.truetype(cFontLatoBold, aFontsize)
    if f != "":
        tw, th = f.getsize(aText);
        th = th + 4
        y = y + ((h - th) / 2)
        x = x + ((b - tw) / 2)
        im.draw.text((x, y), aText, font=f, fill=clWhite)
    x = 2
    y = 2 + h + 2
    f = ImageFont.truetype(cFontLatoLight, 22)
    if f != "":
        if aFaelle7 == '1':
            y = nfCoronaPrintln(im, f, b, y, '%s Corona-Fall in' % aFaelle7)
        else:
            y = nfCoronaPrintln(im, f, b, y, '%s Corona-Fälle in' % aFaelle7)
        y = nfCoronaPrintln(im, f, b, y, 'den letzten 7 Tagen')
        y = nfCoronaPrintln(im, f, b, y, 'in %s' % aOrt)
        y = nfCoronaPrintln(im, f, b, y, 'Stand vom')
        y = nfCoronaPrintln(im, f, b, y, aStandVom)
    if aDateiname != '':
        im.save_png(aDateiname)
    else:
        im.show()


def coronaSammelbildX(im, left, R, aOrt):
    im.rectcl(left, 0, 204, 408, clWhite)
    x = left + 2
    y = 2
    h = 200
    b = 200
    aText = str(R['Inzidenz'])
    aFaelle7 = R['Faelle7']
    aNeu = R['Neu']
    f = ImageFont.truetype(cFontLatoLight, 22)
    if f != "":
        y = nfCoronaPrintln2(im, f, left, b, y, aOrt)
    im.rectcl(x, y, h, b, R['Farbe'])
    f = ImageFont.truetype(cFontLatoBold, 96)
    if f != "":
        tw, th = f.getsize(aText)
        th = th + 4
        y = y + ((h - th) / 2)
        x = x + ((b - tw) / 2)
        im.draw.text((x, y), aText, font=f, fill=clWhite)
    y = 2 + h + 2
    f = ImageFont.truetype(cFontLatoLight, 22)
    if f != "":
        tw, th = f.getsize('Hg')
        y = y + th + 2
        if aNeu == '1':
            y = nfCoronaPrintln2(im, f, left, b, y, '%s neuer Fall' % aNeu)
        else:
            y = nfCoronaPrintln2(im, f, left, b, y, '%s neue Fälle' % aNeu)
        y = y + th + 2
        if aFaelle7 == '1':
            y = nfCoronaPrintln2(im, f, left, b, y, '%s Corona-Fall in' % aFaelle7)
        else:
            y = nfCoronaPrintln2(im, f, left, b, y, '%s Corona-Fälle in' % aFaelle7)
        y = nfCoronaPrintln2(im, f, left, b, y, 'den letzten 7 Tagen')
        y = y + th + 2
    return y, f


def coronaSammelbildXY(im, left, top, R, aOrt):
    im.rectcl(left, top, 204, 330, clWhite)
    x = left + 2
    y = top + 2
    h = 200
    b = 200
    aText = str(R['Inzidenz'])
    aFaelle7 = R['Faelle7']
    aNeu = R['Neu']
    f = ImageFont.truetype(cFontLatoLight, 22)
    if f != "":
        y = nfCoronaPrintlnXYB(im, f, x, y, b, aOrt)
    im.rectcl(x, y, h, b, R['Farbe'])
    if int(aText)>999:
        aFontsize = 84
    else:
        aFontsize = 96
    f = ImageFont.truetype(cFontLatoBold, aFontsize)
    if f != "":
        tw, th = f.getsize(aText)
        th = th + 4
        y = y + ((h - th) / 2)
        x = x + ((b - tw) / 2)
        im.draw.text((x, y), aText, font=f, fill=clWhite)
    x = left + 2
    y = top + 2 + h + 2
    f = ImageFont.truetype(cFontLatoLight, 22)
    if f != "":
        tw, th = f.getsize('Hg')
        y = y + th + 2
        if aNeu == '1':
            y = nfCoronaPrintlnXYB(im, f, x, y, b, '%s neuer Fall' % aNeu)
        else:
            y = nfCoronaPrintlnXYB(im, f, x, y, b, '%s neue Fälle' % aNeu)
        y = y + th + 2
        if aFaelle7 == '1':
            y = nfCoronaPrintlnXYB(im, f, x, y, b, '%s Corona-Fall in' % aFaelle7)
        else:
            y = nfCoronaPrintlnXYB(im, f, x, y, b, '%s Corona-Fälle in' % aFaelle7)
        y = nfCoronaPrintlnXYB(im, f, x, y, b, 'den letzten 7 Tagen')
        y = y + th + 2
    return y, f


def coronaSammelbildQ(aDateiname, nf, lo, zw):
    im = TwnfImage(612, 380, cFontLatoBold, 24, clWhite, clBlack, clBlack)
    y, f = coronaSammelbildX(im, 0, nf, 'Niederfrohna')
    coronaSammelbildX(im, 204, lo, 'Limbach-Oberfrohna')
    coronaSammelbildX(im, 408, zw, 'Kreis Zwickau')
    aStandVom = nf['Datum']
    nfCoronaPrintln(im, f, 612, y, 'Stand vom ' + aStandVom)
    if aDateiname != '':
        im.save_png(aDateiname)
    else:
        im.show()


def coronaSammelbild(aDateiname, nf, lo, zw):
    h = 330
    b = 204
    r = 2
    bi = 2 * b + r + r + r
    hi = 2 * h + r + r + r
    im = TwnfImage(bi, hi, cFontLatoBold, 24, clWhite, clBlack, clBlack)
    im.rectcl(0, 0, bi, hi, clMediumGray)
    coronaSammelbildXY(im, r, r, nf, 'Niederfrohna')
    coronaSammelbildXY(im, b + r + r, r, lo, 'Limbach-Oberfrohna')
    coronaSammelbildXY(im, r, h + r + r, zw, 'Kreis Zwickau')
    aStandVom = nf['Datum']
    im.rectcl(b + r + r, h + r + r, b, h, clWhite)
    f = ImageFont.truetype(cFontLatoLight, 22)
    if f != "":
        nfCoronaPrintlnXYB(im, f, b + r + r, h + r + r + h / 2, b, 'Stand ' + aStandVom)
    if aDateiname != '':
        im.save_png(aDateiname)
    else:
        im.show()


def test():
    im = TwnfImage(600, 400, cFontArial, 10, clWhite, clBlack, clBlack)
    # im.rect(10,10,80,80)
    # im.rectcl(10,10,80,10,wnf_tools.clRed)
    s = "Hg Ostersonntag Hg"
    th = im.TextHeight;
    for i in range(4):
        im.rect(100, 10 + (th * i), 100, th)
        im.text_box_c(100, 10 + (th * i), 100, th, s, clRed)
    ##        im.text_box(10,10+(th*i),80,th,s,wnf_tools.clRed)
    im.rect(100, 100, 80, 60)
    im.text_countdown(100, 100, 80, 60, "50", cFontArial, 24, clLime)
    im.show()


if __name__ == "__main__":
    coronaHochkannt('/tmp/coronaheute.png', clBlue, '9999','1', '12.04.2021','xxxx')
