#!/usr/bin/python3
# -*- coding: utf-8 -*-

import datetime

import caldav
import pytz
import requests
from icalendar import Calendar

import nfix_const as C
import nfix_date as D
import nfix_tools as T

"""
https://pythonhosted.org/caldav/caldav/objects.html
"""


def anzTermine(aCalendar):
    gcal = Calendar.from_ical(aCalendar)
    jetzt = D.wnfJetzt().replace(tzinfo=pytz.UTC)
    # print(jetzt)
    for component in gcal.walk():
        # print(component)
        if component.name == "VEVENT":
            aVon = component.get('dtstart').dt
            aBis = component.get('dtend')
            if aBis:
                aBis = aBis.dt
            if isinstance(aVon, datetime.datetime):
                if aVon > jetzt:
                    print(aVon, aBis, component.get('summary'))
            elif isinstance(aVon, datetime.date):
                if aVon > D.wnfHeute():
                    print(aVon, aBis, component.get('summary'))


def getTermine(aUser):
    ini = T.getIniNfix()
    aSection = C.cSectionRadicale
    aUser = T.leseString(ini, aSection, 'user')
    aUrl = T.leseString(ini, aSection, 'url')
    aPass = T.leseString(ini, aSection, 'pass')
    client = caldav.DAVClient(aUrl, username=aUser, password=aPass)
    principal = client.principal()
    for calendar in principal.calendars():
        # print(type(calendar))
        for event in calendar.events():
            # print(type(event))
            ical_text = event.data
            anzTermine(ical_text)
            # print(ical_text)


def getTermineVonBisEvent(aEvent, aVonX, aBisX):
    gcal = Calendar.from_ical(aEvent)
    jetzt = D.wnfJetzt().replace(tzinfo=pytz.UTC)
    # print(jetzt)
    aResult = []
    for component in gcal.walk():
        # print(component)
        # print('')
        if component.name == "VEVENT":
            aVon = component.get('dtstart').dt
            aBis = component.get('dtend')
            if isinstance(aVon, datetime.datetime):
                aVon = aVon.date()
            if aBis:
                aBis = aBis.dt
                if isinstance(aBis, datetime.date):
                    # Ical liefert als Bis-Datum den nächten Tag,
                    # wenn es sich um einen Ganztagstermin handelt.
                    aBis = aBis - datetime.timedelta(days=1)
                if isinstance(aBis, datetime.datetime):
                    aBis = aBis.date()
                if aBis<aVon:
                    aBis = aVon
            else:
                aBis = None
            aBez = component.get('summary')
            aBez = str(aBez.strip())
            if aVonX < aVon < aBisX:
                aResult = aResult + [(aVon, aBis, aBez)]
    return aResult


def getTermineVonBis(aVon, aBis):
    aTermine = []
    try:
        ini = T.getIniNfix()
        aSection = C.cSectionRadicale
        aUser = T.leseString(ini, aSection, 'user')
        aUrl = T.leseString(ini, aSection, 'url')
        aPass = T.leseString(ini, aSection, 'pass')
        # print(aUrl, aUser)
        client = caldav.DAVClient(aUrl, username=aUser, password=aPass)
        principal = client.principal()
        if principal:
            # print(principal)
            for calendar in principal.calendars():
                # print(calendar)
                for event in calendar.events():
                    # print(event)
                    aTermine = aTermine + getTermineVonBisEvent(event.data, aVon, aBis)
        else:
            print("%s enthält keinen Kalender für User %s" % (aUrl,aUser))
        return aTermine
    except requests.exceptions.ConnectionError as e:
        print(e)
        return aTermine
    except BaseException as e:
        print(type(e))
        print(e)
        return aTermine


def getURL(aURL):
    # r = requests.get(cURL_FritzBox_Test, verify=False)
    r = requests.get(aURL)
    print(r.url)
    print(r.status_code)
    if r.status_code == 200:
        print(r.text)
    else:
        print(r.text)


def main():
    # getURL('https://bone.nfix.de/wetter/48h')
    # getTermine('wnf')
    von = D.ersterDieserWoche(datetime.date.today())
    bis = von + datetime.timedelta(days=28)
    x = getTermineVonBis(von, bis)
    for R in x:
        print(R)


if __name__ == '__main__':
    main()
