#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import configparser
import datetime
import os.path
import pickle
import platform
import shutil
import sys
import tempfile
import termios
import tty
from configparser import ConfigParser
from subprocess import call

import hurry_filesize
# Literatur: http://www.mediasonics.info/_archiv/programming_python/projekt/11.html
import nfix_wallet as W

cNfixIni = 'nfix.ini'
cWnfDesktopKalenderIni = 'wnfDesktopKalender.ini'
cWnfMusikIni = 'wnfMusik.ini'

cVerwandschaft_txt = 'Verwandschaft.txt'
cCeweGeburtstage_csv = 'CeweGeburtstage.csv'

wnfIsWindows = platform.system() == 'Windows'
wnfIsLinux = platform.system() == 'Linux'


def strToIntDef(s):
    try:
        return int(s)
    except ValueError:
        return 0


def int0(i):
    try:
        if int(i) == 0:
            return ''
        else:
            return str(int(i))
    except ValueError:
        return ''


def int2(i):
    try:
        if int(i) == 0:
            return ''
        else:
            return str(int(i)).zfill(2)
    except ValueError:
        return ''


def int4(i):
    try:
        if int(i) == 0:
            return ''
        else:
            return str(int(i)).zfill(4)
    except ValueError:
        return ''


def wnfSaveObject(obj, filename):
    with open(filename, 'wb') as output:  # Overwrites any existing file.
        pickle.dump(obj, output, pickle.HIGHEST_PROTOCOL)


def wnfSaveList(aList, aDateiname):
    with open(aDateiname, 'w') as f:  # Overwrites any existing file.
        for item in aList:
            f.write("%s\n" % item)


def wnfLoadList(aDateiname):
    aList = []
    with open(aDateiname, 'r') as f:
        for line in f:
            aList.append(line.split('\n')[0])
    return aList


def forceDir(aPfad):
    if not os.path.exists(aPfad):
        os.makedirs(aPfad)
    return os.path.exists(aPfad)


def leseString(ini, aSection, aName):
    """" Lesen der Variablen aus der Ini-Datei """
    try:
        s = ini.get(aSection, aName)
    except configparser.NoSectionError:
        s = ""
    except configparser.NoOptionError:
        s = ""
    return s


def schreibeString(ini, aSection, aName, aWert):
    if not ini.has_section(aSection):
        ini.add_section(aSection)
    ini.set(aSection, aName, aWert)


def tempDateiname(aExtension):
    # http://docs.python.org/lib/module-tempfile.html
    # die Datei muss vom Anwender gelöscht werden
    f, s = tempfile.mkstemp(aExtension, "wnf_")
    # da die Datei geöffnet zurückgegeben wird , schließe ich sie jetzt
    os.close(f)
    return s


def wnfHomeDir():
    p = os.path.expanduser('~')
    # return os.environ["HOME"]
    return p


def wnfReparatur_ibus():
    # https://stackoverflow.com/questions/25109770/python-qt-ibus-error-messages
    # https://bugs.launchpad.net/ubuntu/+source/kde-workspace/+bug/855472
    # Bus::open: Can not get ibus-daemon's address.
    # IBusInputContext::createInputContext: no connection to ibus-daemon
    # sudo apt install ibus
    # ibus restart
    # ibus-setup
    aPfad = os.path.join(wnfHomeDir(), '.config', 'ibus', 'bus')
    if not (forceDir(aPfad)):
        print('%s konnte nicht angelegt werden.' % aPfad)


def getHomeDirConfigWlsoft(aDateiname):
    if wnfIsWindows:
        # c:\Users\wnf\AppData\Roaming\wlsoft\
        p = os.path.join(wnfHomeDir(), 'AppData', 'Roaming', 'wlsoft')
    else:
        p = os.path.join(wnfHomeDir(), '.config', 'wlsoft')
    forceDir(p)
    return os.path.join(p, aDateiname)


def getHomeDirConfigWlsoftDir(aDir, aDateiname):
    forceDir(os.path.join(wnfHomeDir(), '.config', 'wlsoft', aDir))
    if wnfIsWindows:
        # c:\Users\wnf\AppData\Roaming\wlsoft\
        p = os.path.join(wnfHomeDir(), 'AppData', 'Roaming', 'wlsoft', aDir)
    else:
        p = os.path.join(wnfHomeDir(), '.config', 'wlsoft', aDir)
    forceDir(p)
    return os.path.join(p, aDateiname)


def getDiskFree(aMount):
    total, used, free = shutil.disk_usage(aMount)
    # return "%s Frei: %s" % (aMount.ljust(30), hurry_filesize.size(free))
    belegt = 100 * used / total
    belegt = round(belegt)
    return aMount, hurry_filesize.size(free), belegt


def getDiskFreeAll():
    s = []
    s = s + [getDiskFree("/")]
    s = s + [getDiskFree("/home")]
    s = s + [getDiskFree("/vmware2")]
    s = s + [getDiskFree("/wnfdaten")]
    return s


def getDiskFreeHtml():
    s = '<table><tr><th>Mount</th><th>Frei</th><th>% Belegt</th></tr>'
    for t1, t2, t3 in getDiskFreeAll():
        s = "%s <tr><td>%s</td><td>%s</td><td>%s %%</td></tr>" % (s, t1, t2, t3)
    s = s + '</table>'
    return s


def getIniNfixDateiname():
    return getHomeDirConfigWlsoft(cNfixIni)


def getIniNfix():
    aIniDateiname = getHomeDirConfigWlsoft(cNfixIni)
    print(aIniDateiname)
    ini = ConfigParser()
    ini.read(aIniDateiname)
    return ini


def getIniDKWallpaper():
    aIniDateiname = getHomeDirConfigWlsoft('nfixDKWallpaper.ini')
    print(aIniDateiname)
    ini = ConfigParser()
    ini.read(aIniDateiname)
    return ini, aIniDateiname


def getIni(aIniDateiname=cNfixIni):
    aIniDateiname = getHomeDirConfigWlsoft(aIniDateiname)
    # print(aIniDateiname)
    ini = configparser.ConfigParser()
    ini.read(aIniDateiname)
    return ini, aIniDateiname


def getIniAnsi(aIniDateiname):
    aIniDateiname = getHomeDirConfigWlsoft(aIniDateiname)
    # print(aIniDateiname)
    ini = configparser.ConfigParser()
    config_file = open(aIniDateiname, encoding="cp1251")
    ini.read_file(config_file)
    return ini, aIniDateiname


def saveIni(ini, aIniDateiname):
    fd = open(aIniDateiname, 'w')
    ini.write(fd)
    fd.close()


def getIBConfDateinameKontakt():
    return getHomeDirConfigWlsoft('wnfKontakt.ini')


def getDatenbankDesktopkalender():
    return getHomeDirConfigWlsoft('wnfDesktopkalender.db')


def getIBConfDateinameKuB():
    return getHomeDirConfigWlsoft('wnfKuB.ini')


def getIBConfDateinameKitaOffice():
    return getHomeDirConfigWlsoft('wnfKitaOffice.ini')


def getPasswort(aIniDateiname, aSection, aName):
    '''
    Das Passwort wird aus der Ini-Datei gelesen und dabei in KWallet gespeichert
    In de Inidatei steht danach cPasswortGespeichert
    und das Passwort wird fürderhin aus dem KWallet gelesen
    :param aIniDateiname:
    :param aSection:
    :param aName:
    :return: Passwort
    '''
    ini, aIniDateiname = getIni(aIniDateiname)
    aPW = leseString(ini, aSection, aName)
    # print(aIniDateiname,aPW)
    if aPW != W.cPasswortGespeichert:
        W.setPasswort(aIniDateiname, aSection, aPW)
        if aPW == (W.getPasswort(aIniDateiname, aSection)):
            schreibeString(ini, aSection, aName, W.cPasswortGespeichert)
            print('%s %s' % (aSection, W.cPasswortGespeichert))
            saveIni(ini, aIniDateiname)
    else:
        aPW = W.getPasswort(aIniDateiname, aSection)
    return aPW


def getFritzPassword():
    return getPasswort(cNfixIni, 'Fritzbox', 'Passwort')


def getFritzIP():
    ini, aIniDateiname = getIni(cNfixIni)
    aIP = leseString(ini, 'Fritzbox', 'IP')
    if aIP == '':
        aIP = '192.168.80.4'
    return aIP


def getMSsqlPassword():
    return getPasswort(cNfixIni, 'MSSQL', 'Passwort')


def wnfMeineXDG(aWas, aDefault):
    # XDG_DESKTOP_DIR="$HOME/Schreibtisch"
    # XDG_DOCUMENTS_DIR="$HOME/Dokumente"
    # XDG_DOWNLOAD_DIR="/wnfdaten/Downloads"
    # XDG_MUSIC_DIR="/wnfdaten/Musik"
    # XDG_PICTURES_DIR="/wnfdaten/Bilder"
    # XDG_PUBLICSHARE_DIR="$HOME/Öffentlich"
    # XDG_TEMPLATES_DIR="$HOME/Vorlagen"
    # XDG_VIDEOS_DIR="/wnfdaten/Videos"

    if wnfIsWindows:
        p = os.path.expanduser('~')
        if aWas == 'XDG_DOCUMENTS_DIR':
            p = os.path.join(p, 'Documents')
        elif (aWas == 'XDG_PICTURES_DIR'):
            p = os.path.join(p, 'Pictures')
        elif (aWas == 'XDG_MUSIC_DIR'):
            p = os.path.join(p, 'Music')
        elif (aWas == 'XDG_DESKTOP_DIR'):
            p = os.path.join(p, 'Desktop')
        else:
            p = os.path.join(p, aDefault)
    else:
        p = os.path.expanduser('~')
        aXDG = os.path.join(p, '.config', 'user-dirs.dirs')
        if os.path.exists(aXDG):
            p = aXDG
            f = open(aXDG, 'r')
            l = f.read().splitlines()
            f.close()
            for z in l:
                if z.startswith(aWas):
                    z = z.split('"')
                    p = z[1]
                    p = os.path.expandvars(p)
    # print(p)
    return p


def wnfMeineBilder():
    return wnfMeineXDG('XDG_PICTURES_DIR', 'pictures')


def wnfMeineMusik():
    return wnfMeineXDG('XDG_MUSIC_DIR', 'music')


def wnfMeineDokumente():
    return wnfMeineXDG('XDG_DOCUMENTS_DIR', 'documents')


def wnfForceLastChar(aString, aChar):
    c = aString[-1]
    if (c != aChar):
        aString = "%s%s" % (aString, aChar)
    return aString


def wnfExePath():
    basis = os.path.dirname(os.path.realpath(__file__))
    # print basis
    return basis


def wnfIsEntwicklung():
    basis = os.path.dirname(os.path.realpath(__file__))
    return (not basis.startswith('/usr'))


def starteKonsole(aProgramm, aParameter):
    # call(['konsole','--hold','-e', 'df','-h'])
    result = call(['konsole', '--hold', '-e', aProgramm, aParameter])
    print(result)


def wnfDirExists(aPfad):
    return os.path.exists(aPfad)


# http://stackoverflow.com/questions/35817/how-to-escape-os-system-calls-in-python
def shellquote(s):
    return "'" + s.replace("'", "'\\''") + "'"


def gwenview(aDateiname):
    wnfProgrammStarten('.', 'gwenview', shellquote(aDateiname))


def kate(aDateiname):
    wnfProgrammStarten('.', 'kate', shellquote(aDateiname))


def wnfFileExists(aDateiname):
    if not aDateiname:
        return False
    b = os.path.exists(aDateiname)
    if b:
        b = os.path.isfile(aDateiname)
    return b


def wnfFileDelete(aDateiname):
    if wnfFileExists(aDateiname):
        os.remove(aDateiname)


def wnfDirDelete(aDirName):
    if wnfDirExists(aDirName):
        shutil.rmtree(aDirName)


def wnfNotFileExistsOrAelter(aQuelle, aZiel):
    b = os.path.exists(aZiel)
    if (b):
        b = os.path.isfile(aZiel)
    if (b):
        aqt = os.path.getctime(aQuelle)
        azt = os.path.getctime(aZiel)
        # print(aqt,azt)
        return azt < aqt
    else:
        return True


def wnfProgrammStarten(aPfad, aProgramm, aParameter):
    if aPfad != '':
        os.chdir(aPfad)
    os.system("%s %s &" % (aProgramm, aParameter))


def wnfDateiStarten(aDateiname):
    if aDateiname.endswith(".bmp"):
        s = "kview %s &" % aDateiname
    elif aDateiname.endswith(".jpg"):
        s = "kview %s &" % aDateiname
    elif aDateiname.endswith(".pdf"):
        if shutil.which('okular'):
            s = "okular %s &" % aDateiname
        elif shutil.which('evince'):
            s = "evince %s &" % aDateiname
        elif shutil.which('xreader'):
            s = "xreader %s &" % aDateiname
        else:
            s = aDateiname
    else:
        s = aDateiname
    return os.system(s)


def wnfFormatPathDir(aDir):
    return wnfForceLastChar(aDir, '/')


# http://stackoverflow.com/questions/35817/how-to-escape-os-system-calls-in-python
def shellQuote(s):
    return "'" + s.replace("'", "'\\''") + "'"


# http://docs.python.org/lib/os-file-dir.html
def isSchreibrecht(aPfad):
    return os.access(aPfad, os.W_OK)


def isLeserecht(aPfad):
    return os.access(aPfad, os.R_OK)


def uptimeSekunden():
    try:
        f = open("/proc/uptime")
        contents = f.read().split()
        f.close()
    except:
        return 0

    return float(contents[0])


# Gives a human-readable uptime string
def uptimeSeit():
    total_seconds = uptimeSekunden()

    # Helper vars:
    MINUTE = 60
    HOUR = MINUTE * 60
    DAY = HOUR * 24

    # Get the days, hours, etc:
    days = int(total_seconds / DAY)
    hours = int((total_seconds % DAY) / HOUR)
    minutes = int((total_seconds % HOUR) / MINUTE)
    seconds = int(total_seconds % MINUTE)

    # Build up the pretty string (like this: "N days, N hours, N minutes, N seconds")
    string = ""
    if days > 0:
        string += str(days) + " " + (days == 1 and "day" or "days") + ", "
    if len(string) > 0 or hours > 0:
        string += str(hours) + " " + (hours == 1 and "hour" or "hours") + ", "
    if len(string) > 0 or minutes > 0:
        string += str(minutes) + " " + (minutes == 1 and "minute" or "minutes") + ", "
    string += str(seconds) + " " + (seconds == 1 and "second" or "seconds")

    return string


def uptimeAmUm():
    total_seconds = uptimeSekunden()
    dt = datetime.datetime.now()
    dt = dt - datetime.timedelta(seconds=total_seconds)
    return dt


def getKommando():
    nb = input('Kommando: ')
    return nb
    # Das funktioniert nicht ...


def getKommando_alt():
    fd = sys.stdin.fileno()
    old_settings = termios.tcgetattr(fd)
    try:
        tty.setraw(sys.stdin.fileno())
        ch = sys.stdin.read(1)
    finally:
        termios.tcsetattr(fd, termios.TCSADRAIN, old_settings)
    return ch


if __name__ == '__main__':
    print(platform.system())
    print(wnfIsWindows)
    print(wnfMeineBilder())
    print(wnfMeineMusik())
    print(wnfMeineDokumente())
    print(uptimeSeit())
    print(uptimeAmUm())
    print(getDiskFreeHtml())
